@@ -1,28 +1,16 @@
 """\
 Perl code generator
 
-How the code is generated: every time the end of an object is reached during
-the parsing of the xml tree, either the function 'add_object' or the function
-'add_class' is called: the latter when the object is a toplevel one, the former
-when it is not. In the last case, 'add_object' calls the appropriate ``writer''
-function for the specific object, found in the 'obj_builders' dict. Such
-function accepts one argument, the CodeObject representing the object for
-which the code has to be written, and returns 3 lists of strings, representing
-the lines to add to the '__init__', '__set_properties' and '__do_layout'
-methods of the parent object.
-
-Like all other perl parts, based on the pre-existing python generators
-
 @copyright: 2002-2004 D.H. aka crazyinsomniac on sourceforge.net
 @copyright: 2012-2016 Carsten Grohmann
-@copyright: 2017 Dietmar Schwertberger
+@copyright: 2017-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import os, os.path, re
-from codegen import BaseLangCodeWriter, BaseSourceFileContent, BaseWidgetHandler
-import wcodegen
-import compat
+from codegen import BaseLangCodeWriter, BaseSourceFileContent
+import wcodegen, compat
+import logging
 
 
 class SourceFileContent(BaseSourceFileContent):
@@ -60,16 +48,13 @@ class SourceFileContent(BaseSourceFileContent):
         r'\s*$'                                 # tailing spaces
         )
 
+    # Regexp to match Perl's Plain Old Documentation format; see: manpage perlpod
     rec_pod = re.compile(
         r'^\s*'                                 # leading spaces
         r'=[A-Za-z_]+\w*'                       # match POD statement
         r'.*$'                                  # any character till eol
         )
-    """\
-    Regexp to match Perl's Plain Old Documentation format
 
-    @see: manpage perlpod
-    """
 
     def build_untouched_content(self):
         """\
@@ -88,6 +73,7 @@ class SourceFileContent(BaseSourceFileContent):
         inside_pod = False
         tmp_in = self._load_file(self.name)
         out_lines = []
+        check_old_methods = []  # list of indices with set_properties or do_layout
         for line in tmp_in:
             result = self.rec_pod.match(line)
             if result:
@@ -101,14 +87,12 @@ class SourceFileContent(BaseSourceFileContent):
             result = self.rec_class_decl.match(line)
             if result:
                 if not self.class_name:
-                    # this is the first class declared in the file: insert the
-                    # new ones before this
+                    # this is the first class declared in the file: insert the new ones before this
                     out_lines.append( '<%swxGlade insert new_classes>' % self.nonce )
                     self.new_classes_inserted = True
                 self.class_name = result.group(1)
                 self.class_name = self.format_classname(self.class_name)
-                self.classes[self.class_name] = 1  # add the found class to the list
-                                              # of classes of this module
+                self.classes.add( self.class_name )  # add the found class to the list of classes of this module
                 out_lines.append(line)
             elif not inside_block:
                 result = self.rec_block_start.match(line)
@@ -126,43 +110,38 @@ class SourceFileContent(BaseSourceFileContent):
                     if not self.class_name:
                         out_lines.append( '<%swxGlade replace %s>' % (self.nonce, which_block) )
                     else:
+                        if which_block in ("__do_layout","__set_properties"):
+                            # probably to be removed
+                            check_old_methods.append( len(out_lines) )
                         out_lines.append( '<%swxGlade replace %s %s>' % (self.nonce, which_class, which_block) )
                 else:
                     result = self.rec_event_handler.match(line)
                     if result:
                         which_handler = result.group('handler')
                         which_class = self.format_classname(result.group('class'))
-                        self.event_handlers.setdefault(
-                            which_class, {})[which_handler] = 1
+                        self.event_handlers.setdefault( which_class, set() ).add( which_handler )
                     if self.class_name and self.is_end_of_class(line):
                         # add extra event handlers here...
                         out_lines.append( '<%swxGlade event_handlers %s>' % (self.nonce, self.class_name) )
                     out_lines.append(line)
-                    if self.is_import_line(line):
-                        # add a tag to allow extra modules
-                        out_lines.append( '<%swxGlade extra_modules>\n' % self.nonce )
             else:
                 # ignore all the lines inside a wxGlade block
                 if self.rec_block_end.match(line):
-##                     self._logger.debug('end block')
                     inside_block = False
         if not self.new_classes_inserted:
             # if we are here, the previous ``version'' of the file did not contain any class, so we must add the
             # new_classes tag at the end of the file
             out_lines.append( '<%swxGlade insert new_classes>' % self.nonce )
+
+        # when moving from 0.9 to 1.0: remove empty methods "do_layout" and "set_properties"
+        while check_old_methods:
+            i = check_old_methods.pop(-1)
+            if out_lines[i+1].strip()=='}':  # just end of block -> remove incl. trailing empty lines
+                self._remove_method(out_lines, i-2, i+1)
+
         # set the ``persistent'' content of the file
         self.content = out_lines
 
-    def is_import_line(self, line):
-        return line.lstrip().startswith('use Wx')
-
-
-
-class WidgetHandler(BaseWidgetHandler):
-    "Interface the various code generators for the widgets must implement"
-    new_signature = []  # Constructor signature ($self->SUPER::new(@stuff)); see PerlCodeWriter.new_defaults
-
-
 
 class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
     "Code writer class for writing Perl code out of the designed GUI elements; see: BaseLangCodeWriter"
@@ -202,9 +181,6 @@ class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
 
     SourceFileContent = SourceFileContent
 
-    tmpl_name_do_layout = '__do_layout'
-    tmpl_name_set_properties = '__set_properties'
-
     tmpl_cfunc_end = '%(tab)sreturn $self;\n' \
                      '\n' \
                      '}\n' \
@@ -213,16 +189,6 @@ class PerlCodeWriter(BaseLangCodeWriter, wcodegen.PerlMixin):
     tmpl_class_end = '\n%(comment)s end of class %(klass)s\n\n1;\n\n'
     tmpl_class_end_nomarker = '\n\n1;\n\n'
 
-    tmpl_ctor_call_layout = '\n' \
-                            '%(tab)s$self->__set_properties();\n' \
-                            '%(tab)s$self->__do_layout();\n\n'
-
-    tmpl_func_do_layout = '\n' \
-                          'sub __do_layout {\n' \
-                          '%(tab)smy $self = shift;\n' \
-                          '%(content)s' \
-                          '}\n'
-
     tmpl_func_event_stub = """\
 
 sub %(handler)s {
@@ -235,12 +201,6 @@ sub %(handler)s {
 
 """
 
-    tmpl_func_set_properties = '\n' \
-                          'sub __set_properties {\n' \
-                          '%(tab)smy $self = shift;\n' \
-                          '%(content)s' \
-                          '}\n'
-
     tmpl_func_empty = '%(tab)sreturn;\n'
 
     tmpl_sizeritem = '%s->Add(%s, %s, %s, %s);\n'
@@ -344,25 +304,23 @@ sub %(handler)s {
         code_lines = []
         write = code_lines.append
 
-        builder = self.obj_builders[code_obj.base]
+        builder = self.obj_builders[code_obj.WX_CLASS]
         mycn = getattr(builder, 'cn', self.cn)
         mycn_f = getattr(builder, 'cn_f', self.cn_f)
 
         # custom base classes support
-        custom_base = getattr( code_obj, 'custom_base', code_obj.properties.get('custom_base', None) )
-        if self.preview or (custom_base and not custom_base.strip()):
-            custom_base = None
+        custom_base = code_obj.check_prop_nodefault('custom_base') and code_obj.custom_base.strip() or None
 
         new_signature = getattr(builder, 'new_signature', [])
 
         # generate constructor code
         if is_new:
             write('package %s;\n\n' % code_obj.klass)
-            write('use Wx qw[:everything];\nuse base qw(%s);\nuse strict;\n\n' % code_obj.base.replace('wx', 'Wx::', 1))
+            write('use Wx qw[:everything];\nuse base qw(%s);\nuse strict;\n\n' % code_obj.WX_CLASS.replace('wx', 'Wx::', 1))
 
             if self._use_gettext:
                 if self.multiple_files:
-                    self.classes[code_obj.klass].dependencies["use Wx::Locale gettext => '_T';\n"] = 1
+                    self.classes[code_obj].dependencies.add( "use Wx::Locale gettext => '_T';\n" )
                 else:
                     write("use Wx::Locale gettext => '_T';\n")
 
@@ -371,7 +329,7 @@ sub %(handler)s {
             # TODO: Don't add dependencies twice with Perl
 
             # write the module dependencies for this class (package)
-            dep_list = sorted( self.classes[code_obj.klass].dependencies.keys() )
+            dep_list = sorted( self.classes[code_obj].dependencies )
             if dep_list:
                 code = self._tagcontent('dependencies', dep_list, True)
                 write(code)
@@ -386,11 +344,10 @@ sub %(handler)s {
                         write(self.new_defaults[k])
             else:
                 new_signature = ['@_[1 .. $#_]']  # shift(@_)->SUPER::new(@_);
-                self._logger.info( "%s did not declare self.new_defaults ", code_obj.klass )
+                logging.info( "%s did not declare self.new_defaults ", code_obj.klass )
 
         elif custom_base:
-            # custom base classes set, but "overwrite existing sources" not
-            # set. Issue a warning about this
+            # custom base classes set, but "overwrite existing sources" not set. Issue a warning about this
             self.warning( '%s has custom base classes, but you are not overwriting existing sources: '
                           'please check that the resulting code is correct!' % code_obj.name )
 
@@ -400,6 +357,11 @@ sub %(handler)s {
                                            'function':self.name_ctor, 'klass':self.cn_class(code_obj.klass),
                                            'tab':tab} )
 
+        # the optional initial code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_pre"):
+            for l in code_obj.properties["extracode_pre"].get_lines():
+                write(tab + l)
+
         style_p = code_obj.properties.get("style")
         if style_p and style_p.value_set != style_p.default_value:
             style = style_p.get_string_value()
@@ -412,56 +374,55 @@ sub %(handler)s {
         write(tab + '$self = $self->SUPER::new( %s );\n' % ", ".join(new_signature))
 
         # set size here to avoid problems with splitter windows
-        if 'size' in code_obj.properties and code_obj.properties["size"].is_active():
-            write( tab+ self.generate_code_size(code_obj) )
+        if code_obj.check_prop('size'):
+            write( tab + self.generate_code_size(code_obj) )
 
-        # classes[code_obj.klass].deps now contains a mapping of child to
-        # parent for all children we processed...
-        object_order = []
-        for obj in self.classes[code_obj.klass].child_order:
-            # Don't add it again if already present
-            if obj in object_order:
-                continue
+        for l in builder.get_properties_code(code_obj):
+            write(tab + l)
 
-            object_order.append(obj)
+        if code_obj.check_prop_truth('extraproperties'):
+            for l in builder.generate_code_extraproperties(code_obj):
+                write(tab + l)
 
-            # Insert parent and ancestor objects before the current object
-            current_object = obj
-            for child, parent in self.classes[code_obj.klass].deps[:]:
-                if child is current_object:
-                    if parent not in object_order:
-                        idx = object_order.index(current_object)
-                        object_order.insert(idx, parent)
-                    current_object = parent
+        # the initial and final code for the contained elements
+        for l in self.classes[code_obj].init:
+            write(tab + l)
+        if self.classes[code_obj].final:
+            write(tab + "\n")
+            for l in self.classes[code_obj].final:
+                write(tab + l)
 
-                    # We processed the dependency: remove it
-                    self.classes[code_obj.klass].deps.remove((child, parent))
+        # now check if there is initial and final code for the element itself
+        for l in builder.get_init_code(code_obj):
+            write(tab+l)
 
-        # Write out the initialisation in the order we just generated
-        for obj in object_order:
-            if obj in self.classes[code_obj.klass].init_lines:
-                for l in self.classes[code_obj.klass].init_lines[obj]:
-                    write(tab + l)
+        for l in builder.get_layout_code(code_obj):
+            write(tab + l)
+
+        # the optional final code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_post"):
+            for l in code_obj.properties["extracode_post"].get_lines():
+                write(tab + l)
 
         return code_lines
 
     def generate_code_event_bind(self, code_obj, tab, event_handlers):
         code_lines = []
-        write = code_lines.append
 
-        for win_id, event, handler, unused in event_handlers:
-            if win_id.startswith('#'):
-                win_id = '$self->{%s}->GetId' % win_id[8:]
+        for obj, event, handler, unused in event_handlers:
+            if obj.name:
+                obj_id = '%s->GetId'%self.format_generic_access(obj) # e.g. '$self->{button_1}->GetId' or '$self->GetId'
+            else:
+                obj_id = self.generate_code_id(None, obj.id)[1] or '-1' # but this is wrong anyway...
 
             if 'EVT_NAVIGATION_KEY' in event:
                 tmpl = '''%(tab)s%(event)s($self, $self->can('%(handler)s'));\n'''
             else:
-                tmpl = '''%(tab)s%(event)s($self, %(win_id)s, $self->can('%(handler)s'));\n'''
-            details = { 'tab': tab, 'event': self.cn(event), 'handler': handler, 'win_id': win_id }
-            write(tmpl % details)
+                tmpl = '''%(tab)s%(event)s($self, %(obj_id)s, $self->can('%(handler)s'));\n'''
+            code_lines.append( tmpl % {'tab': tab, 'event': self.cn(event), 'handler': handler, 'obj_id': obj_id} )
 
         if event_handlers:
-            write('\n')
+            code_lines.append('\n')
 
         return code_lines
 
@@ -469,15 +430,14 @@ sub %(handler)s {
         if id is None:
             id = obj.window_id
         if not id:
-            if obj is not None and "stockitem" in obj.properties and obj.stockitem:
+            if obj is not None and obj.check_prop_truth("stockitem"):
                 return '', self.cn("wxID_" + obj.stockitem)
             return '', self.cn('wxID_ANY')
         id = str(id)
         tokens = id.split('=', 1)
-        if len(tokens) == 2:
-            name, val = tokens
-        else:
+        if len(tokens) != 2:
             return '', self.cn(tokens[0])   # we assume name is declared elsewhere
+        name, val = tokens
         if not name:
             return '', self.cn(val)
         name = name.strip()
@@ -493,35 +453,24 @@ sub %(handler)s {
         objname = self.format_generic_access(obj)
         size = obj.properties["size"].get_string_value()
         use_dialog_units = (size[-1] == 'd')
-        if not obj.parent:
-            method = 'SetSize'
-        else:
-            method = 'SetMinSize'
+        method = 'SetMinSize'  if obj.parent_window  else  'SetSize'
+
         if use_dialog_units:
-            return '%s->%s(%s->ConvertDialogSizeToPixels(Wx::Size->new(%s)));\n' % (
-                   objname,
-                   method,
-                   objname,
-                   size[:-1],
-                   )
-        else:
-            return '%s->%s(Wx::Size->new(%s));\n' % (objname, method, size)
+            return '%s->%s(%s->ConvertDialogSizeToPixels(Wx::Size->new(%s)));\n' % (objname, method, objname, size[:-1])
+        return '%s->%s(Wx::Size->new(%s));\n' % (objname, method, size)
 
     def _quote_str(self, s):
-        """\
-        Escape all unicode characters to there unicode code points in form
-        of \\uxxxx. The returned string is a pure ascii string.
-
+        """Escape all unicode characters to there unicode code points in form of \\uxxxx.
+        The returned string is a pure ascii string.
         Normal ascii characters like \\n or \\t won't be escaped.
 
-        @note: wxGlade don't handles file encoding well currently. Thereby
-               we escape all unicode characters.
+        note: wxGlade don't handles file encoding well currently. Thereby
+              we escape all unicode characters.
 
-        @note: The string 's' is encoded with L{self.app_encoding} already.
+        note: The string 's' is encoded with self.app_encoding already.
 
-        @see: L{BaseLangCodeWriter._quote_str} for additional details
-        @see: L{_recode_x80_xff()}
-        """
+        see: BaseLangCodeWriter._quote_str for additional details
+        see: _recode_x80_xff()"""
         s = s.replace('$', r'\$')
         s = s.replace('@', r'\@')
 
@@ -566,24 +515,22 @@ sub %(handler)s {
         elif obj.name.startswith('$'):
             return obj.name
         # spacer.name is "<width>, <height>" already
-        elif obj.klass == 'spacer':
+        elif obj.WX_CLASS == 'spacer':
             return obj.name
         # Perl stores sizers always in class attributes
-        elif self.store_as_attr(obj) or obj.is_sizer:
+        elif self.store_as_attr(obj) or obj.IS_SIZER:
             return '$self->{%s}' % obj.name
         return '$%s' % obj.name
 
     def _format_import(self, klass):
-        stmt = 'use %s;\n' % klass
-        return stmt
+        return 'use %s;\n' % klass
 
     def _get_class_filename(self, klass):
         "Returns the name for a Perl module (.pm) to store a single class in multi file projects"
-        filename = os.path.join( self.out_dir, klass.replace('::', os.sep) + '.pm' )
-        return filename
+        return os.path.join( self.out_dir, klass.replace('::', os.sep) + '.pm' )
 
     def format_generic_access(self, obj):
-        if obj.is_toplevel:
+        if obj.IS_CLASS:
             return '$self'
         return self._format_classattr(obj)
 
