@@ -1,25 +1,15 @@
 """\
 Python code generator
 
-How the code is generated: every time the end of an object is reached during
-the parsing of the xml tree, either the function 'add_object' or the function
-'add_class' is called: the latter when the object is a toplevel one, the former
-when it is not. In the last case, 'add_object' calls the appropriate ``writer''
-function for the specific object, found in the 'obj_builders' dict. Such
-function accepts one argument, the CodeObject representing the object for
-which the code has to be written, and returns 3 lists of strings, representing
-the lines to add to the '__init__', '__set_properties' and '__do_layout'
-methods of the parent object.
-
 @copyright: John Dubery
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2012-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import os, os.path, random, re
-from codegen import BaseLangCodeWriter, BaseSourceFileContent, BaseWidgetHandler
+from codegen import BaseLangCodeWriter, BaseSourceFileContent
 import wcodegen
 import compat
 
@@ -29,32 +19,31 @@ class SourceFileContent(BaseSourceFileContent):
     rec_block_start = re.compile(
         r'^(?P<spaces>\s*)'                     # leading spaces
         r'#\s*'                                 # comment sign
-        r'begin\s+wxGlade:\s*'                  # "begin wxGlade:" statement and tailing spaces
+        r'begin\s+wxGlade:\s*'                  # "begin wxGlade:" statement and trailing spaces
         r'(?P<classname>[a-zA-Z_]+\w*)??'       # class or function name (non-greedy)
         r'[.]?'                                 # separator between class and function / block (non-greedy)
         r'(?P<block>\w+)'                       # function / block name
-        r'\s*$' )                               # tailing spaces
+        r'\s*$' )                               # trailing spaces
 
     rec_block_end = re.compile(
         r'^\s*'                                 # leading spaces
         r'#\s*'                                 # comment sign
         r'end\s+wxGlade'                        # "end exGlade" statement
-        r'\s*$' )                               # tailing spaces
+        r'\s*$' )                               # trailing spaces
 
     # Less precise regex, but matches definitions with base classes having module qualified names.
     rec_class_decl = re.compile(
-        r'^\s*'                                        # leading spaces
-        r'class\s+([a-zA-Z_]\w*)\s*(\([\s\w.,]*\))?:'  # "class <name>" statement
+        r'^class\s+([a-zA-Z_]\w*)\s*(\([\s\w.,]*\))?:' # starting wihth "class <name>" statement
         r'\s*$' )                                      # tailing spaces
 
     rec_event_handler = re.compile(
         r'^\s+'                                            # leading spaces (mandatory)
         r'def\s+(?P<handler>[A-Za-z_]+\w*)'                # event handler name
         r'\s*'                                             # optional spaces
-        r'\(.*\):'                                         # function parameters
+        r'\(.*\)(?:\s*->\s*None)*:'                        # function parameters; optional PEP 3107 function annotations 
         r'\s*'                                             # optional spaces
         r'#\s*wxGlade:\s*(?P<class>\w+)\.<event_handler>'  # wxGlade event handler statement with class name
-        r'\s*$' )                                          # tailing spaces
+        r'\s*$' )                                          # trailing spaces
 
     def build_untouched_content(self):
         BaseSourceFileContent.build_untouched_content(self)
@@ -63,6 +52,7 @@ class SourceFileContent(BaseSourceFileContent):
         triple_quote_str = None
         tmp_in = self._load_file(self.name)
         out_lines = []
+        check_old_methods = []  # list of indices with __set_properties or __do_layout
         for line in tmp_in:
             if line.endswith("\r\n"):  # normalize line ending for files on Windows
                 line = "%s\n"%line[:-2]
@@ -91,13 +81,12 @@ class SourceFileContent(BaseSourceFileContent):
             result = self.rec_class_decl.match(line)
             if not inside_triple_quote and not inside_block and result:
                 if not self.class_name:
-                    # this is the first class declared in the file: insert the
-                    # new ones before this
+                    # this is the first class declared in the file: insert the new ones before this
                     out_lines.append('<%swxGlade insert new_classes>' % self.nonce)
                     self.new_classes_inserted = True
                 self.class_name = result.group(1)
                 self.class_name = self.format_classname(self.class_name)
-                self.classes[self.class_name] = 1  # add the found class to the list of classes of this module
+                self.classes.add( self.class_name )  # add the found class to the list of classes of this module
                 out_lines.append(line)
             elif not inside_block:
                 result = self.rec_block_start.match(line)
@@ -115,20 +104,20 @@ class SourceFileContent(BaseSourceFileContent):
                     if not self.class_name:
                         out_lines.append('<%swxGlade replace %s>' % (self.nonce, which_block))
                     else:
+                        if which_block in ("__do_layout","__set_properties"):
+                            # probably to be removed
+                            check_old_methods.append( len(out_lines) )
                         out_lines.append('<%swxGlade replace %s %s>' % (self.nonce, which_class, which_block))
                 else:
                     result = self.rec_event_handler.match(line)
                     if not inside_triple_quote and result:
                         which_handler = result.group('handler')
                         which_class = self.format_classname(result.group('class'))
-                        self.event_handlers.setdefault(which_class, {})[which_handler] = 1
+                        self.event_handlers.setdefault( which_class, set() ).add( which_handler )
                     if self.class_name and self.is_end_of_class(line):
                         # add extra event handlers here...
                         out_lines.append('<%swxGlade event_handlers %s>' % (self.nonce, self.class_name))
                     out_lines.append(line)
-                    if self.is_import_line(line):
-                        # add a tag to allow extra modules
-                        out_lines.append('<%swxGlade extra_modules>\n' % self.nonce)
             else:
                 # ignore all the lines inside a wxGlade block
                 if self.rec_block_end.match(line):
@@ -137,12 +126,16 @@ class SourceFileContent(BaseSourceFileContent):
             # if we are here, the previous ``version'' of the file did not  contain any class,
             # so we must add the new_classes tag at the end of the file
             out_lines.append('<%swxGlade insert new_classes>' % self.nonce)
+        
+        # when moving from 0.9 to 1.0: remove empty methods "__do_layout" and "__set_properties"
+        while check_old_methods:
+            i = check_old_methods.pop(-1)
+            if len(out_lines)==i+1 or not out_lines[i+1].strip() or out_lines[i+1].lstrip().startswith("def"):
+                self._remove_method(out_lines, i-1, i)
+
         # set the ``persistent'' content of the file
         self.content = out_lines #"".join(out_lines)
 
-    def is_import_line(self, line):
-        return line.startswith('import wx')
-
     def format_classname(self, class_name):
         """Format class name read from existing source file.
         If we're in a subpackage, we should include the package name in the class name."""
@@ -160,12 +153,6 @@ class SourceFileContent(BaseSourceFileContent):
             return class_name
 
 
-
-class WidgetHandler(BaseWidgetHandler):
-    pass
-
-
-
 class PythonCodeWriter(BaseLangCodeWriter, wcodegen.PythonMixin):
     "Code writer class for writing Python code out of the designed GUI elements"
 
@@ -194,18 +181,19 @@ class PythonCodeWriter(BaseLangCodeWriter, wcodegen.PythonMixin):
 
     name_ctor = '__init__'
 
-    shebang = '#!/usr/bin/env python\n'
+    if compat.PYTHON2:
+        shebang = '#!/usr/bin/env python\n'
+    else:
+        shebang = '#!/usr/bin/env python3\n'
 
     SourceFileContent = SourceFileContent
 
-    tmpl_name_do_layout = '__do_layout'
-    tmpl_name_set_properties = '__set_properties'
     tmpl_encoding = "# -*- coding: %s -*-\n#\n"
     tmpl_class_end = '\n%(comment)s end of class %(klass)s\n'
     tmpl_class_end_nomarker = '\n'
-    tmpl_ctor_call_layout = '\n%(tab)sself.__set_properties()\n%(tab)sself.__do_layout()\n'
     tmpl_func_empty = '%(tab)spass\n'
     tmpl_sizeritem = '%s.Add(%s, %s, %s, %s)\n'
+    tmpl_sizeritem_button = '%s.AddButton(%s)\n'
     tmpl_gridbagsizeritem = '%s.Add(%s, %s, %s, %s, %s)\n'
     if compat.IS_CLASSIC:
         tmpl_gridbagsizerspacer = '%s.Add((%s, %s), %s, %s, %s, %s)\n'
@@ -229,7 +217,7 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
 
         if klass:
             # create application class
-            if top_win and top_win.base=="wxDialog":
+            if top_win and top_win.WX_CLASS=="wxDialog":
                 # use ShowModal()/Destroy for dialogs
                 show_code = ['%(tab)s%(tab)sself.%(top_win)s.ShowModal()',
                             '%(tab)s%(tab)sself.%(top_win)s.Destroy()']
@@ -259,7 +247,7 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
                 ret.append( '%(tab)s%(name)s.MainLoop()' )
             else:
                 # use PySimpleApp
-                if top_win and top_win.base=="wxDialog":
+                if top_win and top_win.WX_CLASS=="wxDialog":
                     show_code = ['%(tab)s%(top_win)s.ShowModal()',
                                  '%(tab)s%(top_win)s.Destroy()']
                 else:
@@ -273,9 +261,6 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
             ret.append('')
         return '\n'.join(ret)
 
-    def __init__(self):
-        BaseLangCodeWriter.__init__(self)
-
     def init_lang(self, app):
         if self.preview and compat.PYTHON2:
             self.header_lines.append('from __future__ import print_function\n')
@@ -290,33 +275,29 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
             if self.multiple_files:
                 self.lang_mapping['import_gettext'] = 'import gettext\n'
             else:
-                self.dependencies['import gettext\n'] = 1
+                self.dependencies.add( 'import gettext\n' )
 
         BaseLangCodeWriter.add_app(self, app, top_win)
 
     def generate_code_ctor(self, code_obj, is_new, tab):
+        # generate code for the class constructor, including all children
         code_lines = []
-        write = code_lines.append
+        write = lambda s: code_lines.append(s if s.strip() else '\n')
 
-        builder = self.obj_builders[code_obj.base]
+        builder = self.obj_builders[code_obj.WX_CLASS]
         mycn = getattr(builder, 'cn', self.cn)
         mycn_f = getattr(builder, 'cn_f', self.cn_f)
-        fmt_klass = self.cn_class(code_obj.klass)
+        fmt_klass = self.cn_class( code_obj.get_prop_value("class", code_obj.WX_CLASS) )
 
         # custom base classes support
-        custom_base = getattr(code_obj, 'custom_base', getattr(code_obj, 'custom_base', None) )
-        if self.preview or (custom_base and not custom_base.strip()):
-            custom_base = None
+        custom_base = None
+        if code_obj.check_prop_nodefault('custom_base') and not self.preview:
+            custom_base = code_obj.custom_base.strip() or None
 
         # generate constructor code
         if is_new:
-            base = mycn(code_obj.base)
-            if custom_base:
-                base = ", ".join([b.strip() for b in custom_base.split(',')])
-            if self.preview and code_obj.klass == base:
-                klass = code_obj.klass + ('_%d' % random.randrange(10 ** 8, 10 ** 9))
-            else:
-                klass = code_obj.klass
+            base = mycn(code_obj.WX_CLASS)
+            if custom_base: base = ", ".join([b.strip() for b in custom_base.split(',')])
             write('\nclass %s(%s):\n' % (self.get_class(fmt_klass), base))
             write(self.tabs(1) + 'def __init__(self, *args, **kwds):\n')
         elif custom_base:
@@ -329,8 +310,13 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
             write(self.tmpl_block_begin % {'class_separator': self.class_separator, 'comment_sign': self.comment_sign,
                                            'function':self.name_ctor, 'klass':fmt_klass, 'tab':tab} )
 
+        # the optional initial code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_pre"):
+            for l in code_obj.properties["extracode_pre"].get_lines():
+                write(tab + l)
+
         style_p = code_obj.properties.get("style")
-        if style_p:# and style_p.value_set != style_p.default_value:
+        if style_p:
             style = style_p.get_string_value()
             m_style = mycn_f( style )
             stmt_style = self._format_style(style, code_obj)
@@ -346,71 +332,72 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
                 else:
                     write(tab + '%s.__init__(self)\n' % b)
         else:
-            write(tab + '%s.__init__(self, *args, **kwds)\n' % mycn(code_obj.base))
+            write(tab + '%s.__init__(self, *args, **kwds)\n' % mycn(code_obj.WX_CLASS))
 
         # set size here to avoid problems with splitter windows
-        if 'size' in code_obj.properties and code_obj.properties["size"].is_active():
+        if code_obj.check_prop('size'):
             write( tab + self.generate_code_size(code_obj) )
 
-        # classes[code_obj.klass].deps now contains a mapping of child to parent for all children we processed...
-        object_order = []
-        for obj in self.classes[code_obj.klass].child_order:
-            # Don't add it again if already present
-            if obj in object_order:
-                continue
+        for l in builder.get_properties_code(code_obj):
+            write(tab + l)
 
-            object_order.append(obj)
+        if not self.preview and code_obj.check_prop_truth('extraproperties'):
+            for l in self.generate_code_extraproperties(code_obj):
+                write(tab + l)
 
-            # Insert parent and ancestor objects before the current object
-            current_object = obj
-            for child, parent in self.classes[code_obj.klass].deps[:]:
-                if child is current_object:
-                    if parent not in object_order:
-                        idx = object_order.index(current_object)
-                        object_order.insert(idx, parent)
-                    current_object = parent
+        # the initial and final code for the contained elements
+        for l in self.classes[code_obj].init:
+            write(tab + l)
+        if self.classes[code_obj].final:
+            write(tab + "\n")
+            for l in self.classes[code_obj].final:
+                write(tab + l)
 
-                    # We processed the dependency: remove it
-                    self.classes[code_obj.klass].deps.remove((child, parent))
+        # now check if there is initial and final code for the element itself
+        for l in builder.get_init_code(code_obj):
+            write(tab+l)
 
-        # Write out the initialisation in the order we just generated
-        for obj in object_order:
-            if obj in self.classes[code_obj.klass].init_lines:
-                for l in self.classes[code_obj.klass].init_lines[obj]:
-                    write(tab + l)
+        for l in builder.get_layout_code(code_obj):
+            write(tab + l)
+
+        # the optional final code from the code properties
+        if not self.preview and code_obj.check_prop("extracode_post"):
+            for l in code_obj.properties["extracode_post"].get_lines():
+                write(tab + l)
 
         return code_lines
 
-    def generate_code_do_layout(self, builder, code_obj, is_new, tab):
-        # Python has two indentation levels
-        #  1st) for function declaration
-        #  2nd) for function body
-        self.tmpl_func_do_layout = '\n' + self.tabs(1) + 'def __do_layout(self):\n%(content)s'
-        return BaseLangCodeWriter.generate_code_do_layout( self, builder, code_obj, is_new, tab )
-
     def generate_code_event_bind(self, code_obj, tab, event_handlers):
         code_lines = []
-        write = code_lines.append
 
-        if event_handlers:
-            write('\n')
+        if event_handlers: code_lines.append('\n')
 
-        for win_id, event, handler, unused in event_handlers:
-            if win_id is None: continue  # bound already, the entry is just for creation of the method stub
-            if win_id.startswith('#'):
-                win_id = win_id[1:]
+        for obj, event, handler, unused in event_handlers:
+            if obj is None: continue  # bound already, the entry is just for creation of the method stub
+            if isinstance(obj, str):
+                obj_name_id = "id=%s"%obj  # mainly for toolbar
             else:
-                win_id = 'id=%s' % win_id
-            
+                obj_name_id = self.format_generic_access(obj)  # e.g. 'self.button_1' or 'self' for toplevels
+
             if not handler.startswith("lambda "):
                 handler = 'self.%s'%handler
 
             if 'EVT_NAVIGATION_KEY' in event:
                 tmpl = '%(tab)sself.Bind(%(event)s, %(handler)s)\n'
             else:
-                tmpl = '%(tab)sself.Bind(%(event)s, %(handler)s, %(win_id)s)\n'
-            details = {'tab':tab, 'event':self.cn(event), 'handler':handler, 'win_id':win_id}
-            write(tmpl % details)
+                tmpl = '%(tab)sself.Bind(%(event)s, %(handler)s, %(obj_name_id)s)\n'
+
+            lines = [tmpl % {'tab':tab, 'event':self.cn(event), 'handler':handler, 'obj_name_id':obj_name_id}]
+
+            if self.preview:
+                # for preview we add an exception handler as the event may be unknown for this wx version or at all
+                indent = self.tabs(1)  # one additional level
+                lines.insert(0, '%stry:\n'%tab)
+                lines[1] = indent + lines[1]  # indent by one level
+                lines.append( '%sexcept AttributeError:\n'%tab )
+                lines.append( '%s%sprint("unknown event %s")\n'%(indent, tab, event) )
+
+            code_lines += lines
 
         return code_lines
 
@@ -434,15 +421,14 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
         if id is None:
             id = obj.window_id
         if not id:
-            if obj is not None and "stockitem" in obj.properties and obj.stockitem:
+            if obj is not None and obj.check_prop_truth("stockitem"):
                 return '', self.cn("wxID_" + obj.stockitem)
             return '', self.cn('wxID_ANY')
         id = str(id)
         tokens = id.split('=', 1)
-        if len(tokens) == 2:
-            name, val = tokens
-        else:
+        if len(tokens) != 2:
             return '', self.cn(tokens[0])   # we assume name is declared elsewhere
+        name, val = tokens
         if not name:
             return '', self.cn(val)
         name = name.strip()
@@ -456,20 +442,12 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
             return '%s = %s\n' % (name, val), name
         return 'global %s; %s = %s\n' % (name, name, val), name
 
-    def generate_code_set_properties(self, builder, code_obj, is_new, tab):
-        # Python has two indentation levels
-        #  1st) for function declaration
-        #  2nd) for function body
-        self.tmpl_func_set_properties = '\n' + self.tabs(1) + 'def __set_properties(self):\n%(content)s'
-
-        return BaseLangCodeWriter.generate_code_set_properties( self, builder, code_obj, is_new, tab )
-
     def generate_code_size(self, obj, size=None):
         objname = self.format_generic_access(obj)
         if size is None:
             size = obj.properties["size"].get_string_value()
         use_dialog_units = (size[-1] == 'd')
-        if not obj.parent:
+        if not obj.parent_window:
             method = 'SetSize'
         else:
             method = 'SetMinSize'
@@ -482,20 +460,17 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
             return '%s.%s((%s))\n' % (objname, method, size)
 
     def _quote_str(self, s):
-        """\
-        Escape all unicode characters to there unicode code points in form
-        of \\uxxxx. The returned string is a pure ascii string.
+        """Escape all unicode characters to there unicode code points in form of \\uxxxx.
+        The returned string is a pure ascii string.
 
         Normal ascii characters like \\n or \\t won't be escaped.
 
-        @note: wxGlade don't handles file encoding well currently. Thereby
-               we escape all unicode characters.
+        note: wxGlade don't handles file encoding well currently. Thereby we escape all unicode characters.
 
-        @note: The string 's' is encoded with L{self.app_encoding} already.
+        note: The string 's' is encoded with self.app_encoding already.
 
-        @see: L{BaseLangCodeWriter._quote_str} for additional details
-        @see: L{_recode_x80_xff()}
-        """
+        see: BaseLangCodeWriter._quote_str for additional details
+            _recode_x80_xff()"""
         # convert all strings to unicode first
         if not isinstance(s, compat.unicode):
             s = s.decode(self.app_encoding)
@@ -535,26 +510,23 @@ from %(top_win_module)s import %(top_win_class)s\n\n"""
         elif obj.name.startswith('self.'):
             return obj.name
         # spacer.name is "<width>, <height>" already, but wxPython expect a tuple instead of two single values
-        elif obj.klass in ('spacer','sizerslot'):
+        elif obj.WX_CLASS in ('spacer','sizerslot'):
             return '(%s)' % obj.name
         elif self.store_as_attr(obj):
             return 'self.%s' % obj.name
         return obj.name
 
     def _format_import(self, klass):
-        stmt = 'from %s import %s\n' % (klass, self.get_class(klass))
-        return stmt
+        return 'from %s import %s\n' % (klass, self.get_class(klass))
 
     def _get_class_filename(self, klass):
-        filename = os.path.join( self.out_dir, klass.replace('.', os.sep) + '.py' )
-        return filename
+        return os.path.join( self.out_dir, klass.replace('.', os.sep) + '.py' )
 
     def format_generic_access(self, obj):
-        if obj.is_toplevel:
+        if obj.IS_CLASS:
             return 'self'
-        else:
-            return self._format_classattr(obj)
+        return self._format_classattr(obj)
 
 
-writer = PythonCodeWriter()  # The code writer is an instance of L{PythonCodeWriter}.
+writer = PythonCodeWriter()  # The code writer is an instance of PythonCodeWriter.
 language = writer.language   # Language generated by this code generator
