@@ -2,7 +2,7 @@
 Compatibility code to run with different versions of wxPython
 
 @copyright: 2013-2016 Carsten Grohmann
-@copyright: 2017-2018 Dietmar Schwertberger
+@copyright: 2017-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
@@ -40,18 +40,14 @@ def SizerItem_AssignWindow(item, window):
     """
     Wrapper for wxSizerItem.SetWindow() resp. wxSizerItem.AssignWindow()
 
-    wxSizerItem.SetWindow() is deprecated since wxPython 2.9 use
-    wxSizerItem.AssignWindow() instead.
+    wxSizerItem.SetWindow() is deprecated since wxPython 2.9 use wxSizerItem.AssignWindow() instead.
 
-    Depending on the wxPython version L{SizerItem_SetWindow28()} or
-    L{SizerItem_AssignWindow()} will be used.
+    Depending on the wxPython version SizerItem_SetWindow28() or SizerItem_AssignWindow() will be used.
 
-    @param item:   Instance of wxSizerItem
-    @param window: Instance of wxWindow
+    item:   Instance of wxSizerItem
+    window: Instance of wxWindow
 
-    @see: L{SizerItem_SetWindow()}
-    @see: L{SizerItem_SetWindow28()}
-    @see: L{SizerItem_AssignWindow()}
+    see: SizerItem_SetWindow(), SizerItem_SetWindow28(), SizerItem_AssignWindow()
     """
     item.AssignWindow(window)
 
@@ -60,19 +56,14 @@ def SizerItem_SetWindow28(item, window):
     """
     Wrapper for wxSizerItem.SetWindow() resp. wxSizerItem.AssignWindow()
 
-    wxSizerItem.SetWindow() is deprecated since wxPython 2.9 use
-    wxSizerItem.AssignWindow() instead.
+    wxSizerItem.SetWindow() is deprecated since wxPython 2.9 use wxSizerItem.AssignWindow() instead.
 
-    Depending on the wxPython version L{SizerItem_SetWindow28()} or
-    L{SizerItem_AssignWindow()} will be used.
+    Depending on the wxPython version SizerItem_SetWindow28() or SizerItem_AssignWindow() will be used.
 
-    @param item:   Instance of wxSizerItem
-    @param window: Instance of wxWindow
+    item:   Instance of wxSizerItem
+    window: Instance of wxWindow
 
-    @see: L{SizerItem_SetWindow()}
-    @see: L{SizerItem_SetWindow28()}
-    @see: L{SizerItem_AssignWindow()}
-    """
+    see: SizerItem_SetWindow(), SizerItem_SetWindow28(), SizerItem_AssignWindow()"""
     item.SetWindow(window)
 
 
@@ -80,12 +71,10 @@ def SizerItem_SetSizer(item, sizer):
     """
     Wrapper for wxSizerItem.SetSizer() resp. wxSizerItem.AssignSizer()
 
-    wxSizerItem.SetSizer() is deprecated since wxPython 2.9 use
-    wxSizerItem.AssignSizer() instead.
+    wxSizerItem.SetSizer() is deprecated since wxPython 2.9 use wxSizerItem.AssignSizer() instead.
 
-    @param item:  Instance of wxSizerItem
-    @param sizer: Instance of wxSizer
-    """
+    item:  Instance of wxSizerItem
+    sizer: Instance of wxSizer"""
     # don't use for now, as this causes crashes in e.g. change_sizer
     #if hasattr(item, 'AssignSizer'):
         #item.AssignSizer(sizer)
@@ -206,6 +195,27 @@ else:
         widget.Hide()
         wx.CallAfter(_Destroy, widget)
 
+def wxWindow_SendSizeEventToParent28(item, flags=0):
+    """\
+    Implementation of wxWindow.SendSizeEventToParent for wxPython 2.8
+
+    @param item:  Instance of wxWindow
+    @param flags: flags parameter for SendSizeEvent (ignored)
+    """
+    parent = item.GetParent()
+    if parent and not parent.IsBeingDeleted():
+        parent.SendSizeEvent()
+
+
+def wxWindow_SendSizeEventToParent3(item, flags=0):
+    """\
+    Wrapper of wxWindow.SendSizeEventToParent
+
+    @param item:  Instance of wxWindow
+    @param flags: flags parameter for SendSizeEvent
+    """
+    item.SendSizeEventToParent(flags)
+
 
 # Set different functions depending on the active wxPython version
 if wx.VERSION[:2] >= (2, 9):
@@ -213,11 +223,13 @@ if wx.VERSION[:2] >= (2, 9):
     GridSizer_GetCols = GridSizer_GetCols3
     SizerItem_SetWindow = SizerItem_AssignWindow
     wxWindow_IsEnabled = wxWindow_IsThisEnabled
+    wxWindow_SendSizeEventToParent = wxWindow_SendSizeEventToParent3
 else:
     GridSizer_GetRows = GridSizer_GetRows28
     GridSizer_GetCols = GridSizer_GetCols28
     SizerItem_SetWindow = SizerItem_SetWindow28
     wxWindow_IsEnabled = wxWindow_IsEnabled28
+    wxWindow_SendSizeEventToParent = wxWindow_SendSizeEventToParent28
 
 
 import wx.grid
@@ -243,6 +255,7 @@ if len(wx.VERSION)==5:
     def SetToolTip(c, s):
         c.SetToolTipString(s)
     wx_EmptyBitmap = wx.EmptyBitmap
+    wx_EmptyImage = wx.EmptyImage
     wx_EmptyIcon = wx.EmptyIcon
     wx_NamedColour = wx.NamedColour
     def ConvertPixelsToDialog(widget, size):
@@ -250,6 +263,8 @@ if len(wx.VERSION)==5:
 
     ListCtrl_SetStringItem    = wx.ListCtrl.SetStringItem
     ListCtrl_InsertStringItem = wx.ListCtrl.InsertStringItem
+    ListCtrl_InsertImageItem = wx.ListCtrl.InsertImageItem
+
     def SetCursor(window, cursor):
         window.SetCursor(wx.StockCursor(cursor))
         
@@ -273,6 +288,7 @@ else:
     def SetToolTip(c, s):
         c.SetToolTip(s)
     wx_EmptyBitmap = wx.Bitmap
+    wx_EmptyImage = wx.Image
     wx_EmptyIcon = wx.Icon
     wx_NamedColour = wx.Colour
     def ConvertPixelsToDialog(widget, size):
@@ -280,6 +296,8 @@ else:
 
     ListCtrl_SetStringItem    = wx.ListCtrl.SetItem
     ListCtrl_InsertStringItem = wx.ListCtrl.InsertItem
+    ListCtrl_InsertImageItem = wx.ListCtrl.InsertItem
+
     def SetCursor(window, cursor):
         window.SetCursor(wx.Cursor(cursor))
 
