@@ -7,17 +7,14 @@ Code shared between different language code generators
 
 
 class CheckBoxMixin(object):
-    """\
-    Generic code to handle wxCheckbox code in all language code generators
-    """
+    "Generic code to handle wxCheckbox code in all language code generators"
 
     def _prepare_checkbox_content(self, obj):
-        """\
+        """
         Prepare template variables for 3-state checkbox
 
-        @param obj: Instance of L{xml_parse.CodeObject}
-        @type obj: xml_parse.CodeObject
-        """
+        obj: Instance of xml_parse.CodeObject
+        obj: xml_parse.CodeObject"""
         checked = obj.checked
         if self.cn_f('wxCHK_3STATE') in self.tmpl_dict['style']:
             checked = self.config['number2state'][checked]
@@ -26,13 +23,8 @@ class CheckBoxMixin(object):
         else:
             self.has_setvalue1 = checked
 
-    def _get_checkbox_code(self, prop_lines):
-        """\
-        Add code to set the state of a 3-state checkbox to prop_lines.
-
-        @param prop_lines:
-        @type prop_lines: list
-        """
+    def _get_checkbox_code(self):
+        "Returns code to set the state of a 3-state checkbox to prop_lines."
         if self.cn_f('wxCHK_3STATE') in self.tmpl_dict['style']:
-            prop_lines.append(self.tmpl_set3statevalue % self.tmpl_dict)
-        return
+            return [self.tmpl_set3statevalue % self.tmpl_dict]
+        return []
