@@ -3,13 +3,12 @@ Custom wxWindow objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2017-2019 Dietmar Schwertberger
+@copyright: 2017-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import wx
 import common, misc
-from tree import Node
 from wcodegen.taghandler import BaseXmlBuilderTagHandler
 import new_properties as np
 from edit_windows import ManagedBase
@@ -40,8 +39,7 @@ class ArgumentsHandler(BaseXmlBuilderTagHandler):
 
     def end_elem(self, name):
         if name == 'arguments':
-            self.parent.properties['arguments'].set(self.arguments)
-            self.parent.properties_changed(["arguments"])
+            self.parent.properties['arguments'].load(self.arguments)
             return True
         elif name == 'argument':
             char_data = self.get_char_data()
@@ -56,14 +54,19 @@ class CustomWidget(ManagedBase):
     arguments: Constructor arguments
     custom_ctor: if not empty, an arbitrary piece of code that will be used instead of the constructor name"""
 
+    WX_CLASS = "CustomWidget"
     _PROPERTIES = ["Widget", "custom_ctor", "arguments"]
     PROPERTIES = ManagedBase.PROPERTIES + _PROPERTIES + ManagedBase.EXTRA_PROPERTIES
 
     _PROPERTY_LABELS = { 'custom_constructor':'Custom constructor' }
-    _PROPERTY_HELP   = { 'custom_constructor':'Specify a custom constructor like a factory method' }
+    _PROPERTY_HELP   = { 'custom_constructor':'Specify a custom constructor like a factory method',
+                         "instance_class":("The class that should be instantiated, e.g. 'mycontrols.MyCtrl'.\n\n"
+                            "You need to ensure that the class is available.\n"
+                            "Add required import code to 'Extra (import) code for this widget' on the Code tab.")}
 
-    def __init__(self, name, klass, parent, id, sizer, pos):
-        ManagedBase.__init__(self, name, klass, parent, id, sizer, pos)
+    def __init__(self, name, parent, index, instance_class=None):
+        ManagedBase.__init__(self, name, parent, index, instance_class or "wxWindow")
+        self.properties["instance_class"].deactivated = None
 
         # initialise instance properties
         cols      = [('Arguments', np.GridProperty.STRING)]
@@ -71,12 +74,12 @@ class CustomWidget(ManagedBase):
         self.custom_ctor = np.TextPropertyD("", name="custom_constructor", strip=True, default_value="")
 
     def create_widget(self):
-        self.widget = wx.Window( self.parent.widget, self.id, style=wx.BORDER_SUNKEN | wx.FULL_REPAINT_ON_RESIZE)
+        self.widget = wx.Window(self.parent_window.widget, wx.ID_ANY, style=wx.BORDER_SUNKEN | wx.FULL_REPAINT_ON_RESIZE)
         self.widget.Bind(wx.EVT_PAINT, self.on_paint)
         self.widget.Bind(wx.EVT_ERASE_BACKGROUND, lambda event:None)
 
-    def finish_widget_creation(self):
-        ManagedBase.finish_widget_creation(self, sel_marker_parent=self.widget)
+    def finish_widget_creation(self, level):
+        ManagedBase.finish_widget_creation(self, level, sel_marker_parent=self.widget)
 
     def on_paint(self, event):
         dc = wx.PaintDC(self.widget)
@@ -87,7 +90,7 @@ class CustomWidget(ManagedBase):
         w, h = self.widget.GetClientSize()
         dc.DrawLine(0, 0, w, h)
         dc.DrawLine(w, 0, 0, h)
-        text = _('Custom Widget: %s') % self.klass
+        text = _('Custom Widget: %s') % self.instance_class
         tw, th = dc.GetTextExtent(text)
         x = (w - tw)//2
         y = (h - th)//2
@@ -100,11 +103,10 @@ class CustomWidget(ManagedBase):
             return ArgumentsHandler(self)
         return ManagedBase.get_property_handler(self, name)
 
-    def properties_changed(self, modified):
-        if not modified or "klass" in modified:
-            if self.widget:
-                self.widget.Refresh()
-        ManagedBase.properties_changed(self, modified)
+    def _properties_changed(self, modified, actions):
+        if modified and "instance_class" in modified:
+            actions.update(("refresh","label"))
+        ManagedBase._properties_changed(self, modified, actions)
 
 
 class Dialog(wx.Dialog):
@@ -112,14 +114,13 @@ class Dialog(wx.Dialog):
     validation_re  = re.compile(r'^[a-zA-Z_\.]+[\w-]*(\[\w*\])*$')  # does not avoid ".."
     def __init__(self):
         title = _('Enter widget class')
-        pos = wx.GetMousePosition()
-        wx.Dialog.__init__(self, None, -1, title, pos)
+        wx.Dialog.__init__(self, None, -1, title, wx.GetMousePosition())
         klass = 'CustomWidget'
 
         self.classname = wx.TextCtrl(self, -1, klass)
         sizer = wx.BoxSizer(wx.VERTICAL)
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
-        hsizer.Add(wx.StaticText(self, -1, _('class')), 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
+        hsizer.Add(wx.StaticText(self, -1, _('Instance class')), 0, wx.ALL | wx.ALIGN_CENTER_VERTICAL, 5)
         hsizer.Add(self.classname, 1, wx.ALIGN_CENTER_VERTICAL|wx.TOP|wx.BOTTOM|wx.RIGHT, 3)
         sizer.Add(hsizer, 0, wx.EXPAND)
 
@@ -146,7 +147,7 @@ class Dialog(wx.Dialog):
         self.OK_button.Enable( OK )
 
 
-def builder(parent, sizer, pos, number=[1]):
+def builder(parent, index):
     "factory function for CustomWidget objects"
 
     dialog = Dialog()
@@ -156,47 +157,26 @@ def builder(parent, sizer, pos, number=[1]):
     dialog.Destroy()
     if res != wx.ID_OK: return
 
-    name = 'window_%d' % number[0]
-    while common.app_tree.has_name(name):
-        number[0] += 1
-        name = 'window_%d' % number[0]
+    name = parent.toplevel_parent.get_next_contained_name('window_%d')
     with parent.frozen():
-        win = CustomWidget(name, klass, parent, wx.NewId(), sizer, pos)
-        node = Node(win)
-        win.node = node
-
-        win.properties["arguments"].set( [['$parent'], ['$id']] )  # ,['$width'],['$height']]
-        win.properties["proportion"].set(1)
-        win.properties["flag"].set("wxEXPAND")
-        if parent.widget: win.create()
-
-    common.app_tree.insert(node, sizer.node, pos-1)
-    #sizer.set_item(win.pos, 1, wx.EXPAND)
+        editor = CustomWidget(name, parent, index, klass)
+        editor.properties["arguments"].set( [['$parent'], ['$id']] )  # ,['$width'],['$height']]
+        editor.properties["proportion"].set(1)
+        editor.properties["flag"].set("wxEXPAND")
+        if parent.widget: editor.create()
+    return editor
 
 
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory to build CustomWidget objects from a XML file"
-    from xml_parse import XmlParsingError
-    try:
-        name = attrs['name']
-    except KeyError:
-        raise XmlParsingError(_("'name' attribute missing"))
-    if not sizer or not sizeritem:
-        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
-    klass = attrs.get("class", "CustomWidget")
-    win = CustomWidget(name, klass, parent, wx.NewId(), sizer, pos)
-    #sizer.set_item(win.pos, proportion=sizeritem.proportion, span=sizeritem.span, flag=sizeritem.flag, border=sizeritem.border)
-    node = Node(win)
-    win.node = node
-    if pos is None: common.app_tree.add(node, sizer.node)
-    else: common.app_tree.insert(node, sizer.node, pos-1)
-    return win
+    return CustomWidget(name, parent, index)
 
 
 def initialize():
     "initialization function for the module: returns a wx.BitmapButton to be added to the main palette"
+    common.widget_classes['CustomWidget'] = CustomWidget
     common.widgets['CustomWidget'] = builder
     common.widgets_from_xml['CustomWidget'] = xml_builder
 
-    return common.make_object_button('CustomWidget', 'custom.xpm', tip='Add a custom widget')
+    return common.make_object_button('CustomWidget', 'custom.png', tip='Add a custom widget')
 
