@@ -12,17 +12,13 @@ import wcodegen
 
 
 class LispGaugeGenerator(wcodegen.LispWidgetCodeWriter):
-
-    tmpl = '(setf %(name)s (%(klass)s_Create %(parent)s %(id)s ' \
-           '%(range)s -1 -1 -1 -1 %(style)s))\n'
+    tmpl = '(setf %(name)s (%(klass)s_Create %(parent)s %(id)s %(range)s -1 -1 -1 -1 %(style)s))\n'
 
     def _prepare_tmpl_content(self, obj):
         wcodegen.LispWidgetCodeWriter._prepare_tmpl_content(self, obj)
-        self.tmpl_dict['range'] = obj.properties.get('range', '10')
+        self.tmpl_dict['range'] = obj.range
         return
 
-# end of class LispGaugeGenerator
-
 
 def initialize():
     klass = 'wxGauge'
