@@ -3,15 +3,13 @@ wxPanel objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import logging
 import wx
-import clipboard
-import common, compat, config, misc
-from tree import Node
+import common, config, misc, clipboard
 import new_properties as np
 from edit_windows import ManagedBase, TopLevelBase, EditStylesMixin
 
@@ -19,37 +17,42 @@ from edit_windows import ManagedBase, TopLevelBase, EditStylesMixin
 
 class PanelBase(EditStylesMixin):
     "Class PanelBase"
-    _custom_base_classes = True
 
-    _PROPERTIES = ["Widget", "no_custom_class", "style", "scrollable", "scroll_rate"]
-    _PROPERTY_LABELS = {'no_custom_class':"Don't generate code for this class"}
-    _PROPERTY_HELP = {'no_custom_class':'If this is a custom class, setting this property prevents wxGlade\n'
-                                        'from generating the class definition code'}
+    _PROPERTIES = ["Widget", "style", "scrollable", "scroll_rate"]
+    CHILDREN = -1  # 0 or 1; either a sizer or nothing
+    TREE_ICON = "EditPanel"
 
     def __init__(self, style='wxTAB_TRAVERSAL'):
         "Class to handle wxPanel objects"
-        # initialise instance logger
-        self._logger = logging.getLogger(self.__class__.__name__)
-
-        # initialise instance
         EditStylesMixin.__init__(self, 'wxPanel')
-        self.top_sizer = None  # sizer to handle the layout of children
 
         # initialise properties
-        self.no_custom_class = np.CheckBoxProperty(False, default_value=False)
         self.scrollable      = np.CheckBoxProperty(False, default_value=False)
         self.scroll_rate = prop = np.IntPairPropertyD( "10, 10" )
         prop.set_blocked(True)
 
         if style: self.properties["style"].set(style)
 
-    def finish_widget_creation(self):
-        super(PanelBase, self).finish_widget_creation(sel_marker_parent=self.widget)
+    def get_editor_name(self):
+        ret = self.__class__.__name__
+        if self.WX_CLASS=="wxScrolledWindow":
+            # self.scrolled is True
+            # "EditPanel"         -> 'EditScrolledWindow'
+            # "EditTopLevelPanel" -> "EditTopLevelScrolledWindow"
+            ret = ret.replace("Panel", "ScrolledWindow")
+        return ret
+
+    def finish_widget_creation(self, level):
+        if self.IS_TOPLEVEL:
+            super(PanelBase, self).finish_widget_creation(level)
+        else:
+            super(PanelBase, self).finish_widget_creation(level, sel_marker_parent=self.widget)
+
         if self.scrollable:
             self.widget.SetScrollRate( *self.properties["scroll_rate"].get_tuple() )
         # this must be done here since ManagedBase.finish_widget_creation normally sets EVT_LEFT_DOWN to update_view
         if not self.widget.Disconnect(-1, -1, wx.wxEVT_LEFT_DOWN):
-            self._logger.warning( _("EditPanel: Unable to disconnect the event handler") )
+            logging.warning( _("EditPanel: Unable to disconnect the event handler") )
         self.widget.Bind(wx.EVT_LEFT_DOWN, self.drop_sizer)
 
     def _update_markers(self, event):
@@ -65,45 +68,47 @@ class PanelBase(EditStylesMixin):
         event.Skip()
 
     def on_enter(self, event):
-        if not self.top_sizer and common.adding_sizer:
+        if self.children and common.adding_sizer:
             self.widget.SetCursor(wx.CROSS_CURSOR)
         else:
             self.widget.SetCursor(wx.STANDARD_CURSOR)
 
     def set_sizer(self, sizer):
-        self.top_sizer = sizer
-        if self.top_sizer and self.top_sizer.widget and self.widget:
+        if sizer and sizer.widget and self.widget:
             self.widget.SetAutoLayout(True)
-            self.widget.SetSizer(self.top_sizer.widget)
+            self.widget.SetSizer(sizer.widget)
             self.widget.Layout()
-        elif self.top_sizer is None and self.widget:
+        elif sizer is None and self.widget:
             self.widget.SetSizer(None)
 
-    def drop_sizer(self, event=None):
-        if self.top_sizer or not common.adding_sizer:
+    def drop_sizer(self, event=None, reset=None):
+        if self.children or not common.adding_sizer:
             self.on_set_focus(event)  # default behaviour: call show_properties
             return
         if self.widget: self.widget.SetCursor(wx.NullCursor)
-        common.widgets[common.widget_to_add](self, None, None)
+        new_widget = common.widgets[common.widget_to_add](self, None)
+        if new_widget is None: return
+        misc.rebuild_tree(new_widget)
+        if reset is False: return
         if event is None or not misc.event_modifier_copy(event):
             common.adding_widget = common.adding_sizer = False
             common.widget_to_add = None
-        common.app_tree.app.saved = False
 
     def check_drop_compatibility(self):
-        if self.top_sizer:
-            return (False, 'Sizer already set for this panel')
+        if self.children:
+            return (False, 'Sizer or child widget already set for this panel.')
+
         if common.adding_sizer:
             return (True, None)
-        return (False, 'Only sizers can be added here')
+        return (False, 'Only sizers can be added here; optionally, delete the panel and add e.g. a notebook. ')
 
     def get_widget_best_size(self):
-        if self.top_sizer and self.widget.GetSizer():
-            self.top_sizer.fit_parent()
+        if self.children and self.widget.GetSizer():
+            self.children[0].fit_parent()
             return self.widget.GetSize()
         return self.widget.__class__.GetBestSize(self.widget)
 
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         if not modified or "scrollable" in modified:
             if self.scrollable:
                 if self.klass == 'wxPanel':
@@ -118,42 +123,32 @@ class PanelBase(EditStylesMixin):
 
         if self.widget and modified:
             if "scrollable" in modified and self.properties["scrollable"].previous_value!=self.scrollable:
-                self.recreate_widget()
+                actions.add("recreate")
             elif "scroll_rate" in modified and self.scrollable and isinstance(self.widget, wx.ScrolledWindow):
                 self.widget.SetScrollRate( *self.properties["scroll_rate"].get_tuple() )
-        EditStylesMixin.properties_changed(self, modified)
-
-    def __getstate__(self):
-        state = self.__dict__.copy()
-        del state['_logger']
-        return state
-
-    def __setstate__(self, state):
-        self.__dict__.update(state)
-
-        # re-initialise logger instance deleted from __getstate__
-        self._logger = logging.getLogger(self.__class__.__name__)
-
+        EditStylesMixin._properties_changed(self, modified, actions)
 
 
 class EditPanel(PanelBase, ManagedBase):
     "Class to handle wxPanel objects"
-
+    WX_CLASS = "wxPanel"  # this will be overwritten in properties_changed depending on the "scrolled" property
+    WX_CLASSES = ("wxPanel", "wxScrolledWindow")
     PROPERTIES = ManagedBase.PROPERTIES + PanelBase._PROPERTIES + ManagedBase.EXTRA_PROPERTIES
+    np.insert_after(PROPERTIES, "name", "class", "custom_base")
 
-    def __init__(self, name, parent, id, sizer, pos, style='wxTAB_TRAVERSAL'):
-        ManagedBase.__init__(self, name, 'wxPanel', parent, id, sizer, pos)
+    def __init__(self, name, parent, index, style='wxTAB_TRAVERSAL'):
+        ManagedBase.__init__(self, name, parent, index)
         PanelBase.__init__(self, style)
 
     def create_widget(self):
         # to be done: use ScrolledWindow only if scrolling is required
         if self.scrollable:
-            self.widget = wx.ScrolledWindow(self.parent.widget, self.id, style=0)
+            self.widget = wx.ScrolledWindow(self.parent_window.widget, wx.ID_ANY, style=0)
         else:
-            self.widget = wx.Panel(self.parent.widget, self.id, style=0)
+            self.widget = wx.Panel(self.parent_window.widget, wx.ID_ANY, style=0)
         self.widget.Bind(wx.EVT_ENTER_WINDOW, self.on_enter)
         self.widget.GetBestSize = self.get_widget_best_size
-        if self.sizer.is_virtual():
+        if not self.parent.IS_SIZER:
             def GetBestSize():
                 if self.widget and self.widget.GetSizer():
                     return self.widget.GetSizer().GetMinSize()
@@ -161,9 +156,11 @@ class EditPanel(PanelBase, ManagedBase):
             self.widget.GetBestSize = GetBestSize
 
     def set_sizer(self, sizer):
+        # called from sizer.create: self.window.set_sizer(self)
+        assert self.children and sizer is self.children[0]
         super(EditPanel, self).set_sizer(sizer)
-        if self.top_sizer and self.top_sizer.widget and self.widget:
-            self.sizer.set_item_best_size(self, size=self.widget.GetBestSize())
+        if sizer and sizer.widget and self.widget:
+            sizer.set_item_best_size(self, size=self.widget.GetBestSize())
 
     def _create_popup_menu(self, widget=None):
         if widget is None: widget = self.widget
@@ -177,11 +174,11 @@ class EditPanel(PanelBase, ManagedBase):
 
         i = misc.append_menu_item(menu, -1, _('Paste Sizer\tCtrl+V'), wx.ART_PASTE)
         misc.bind_menu_item_after(widget, i, clipboard.paste, self)
-        if self.top_sizer is not None or not clipboard.check("sizer"): i.Enable(False)
+        if self.children or not clipboard.check("sizer"): i.Enable(False)
         menu.AppendSeparator()
 
-        if self.sizer and hasattr(self.sizer, "_add_popup_menu_items"):
-            self.sizer._add_popup_menu_items(menu, self, widget)
+        if hasattr(self.parent, "_add_parent_popup_menu_items"):
+            self.parent._add_parent_popup_menu_items(menu, self, widget)
 
         i = misc.append_menu_item(menu, -1, _('Preview'))
         misc.bind_menu_item_after(widget, i, self.preview_parent)
@@ -190,52 +187,51 @@ class EditPanel(PanelBase, ManagedBase):
 
     def check_compatibility(self, widget, typename=None, report=True):
         "check whether widget can be pasted"
+        if self.children and not self.children[0].IS_SLOT:
+            return (False, 'Sizer already set for this panel')
         if typename is not None:
             if typename!="sizer":
                 return (False, 'Only sizers can be pasted here')
-            if self.top_sizer:
-                return (False, 'Sizer set already')
             return (True, None)
 
-        import edit_sizers
-        if not isinstance(widget, edit_sizers.Sizer):
+        if not widget.IS_SIZER:
             return (False, 'Only sizers can be pasted here')
-        #if self.sizer is not None:
-        if self.top_sizer:
-            return (False, 'Sizer set already')
         return (True, None)
 
     def clipboard_paste(self, clipboard_data):
         "Insert a widget from the clipboard to the current destination"
         if self.widget: size = self.widget.GetSize()
-        ret = clipboard._paste(self, None, 0, clipboard_data)
+        ret = clipboard._paste(self, 0, clipboard_data)
         if self.widget: self.widget.SetSize(size)
         return ret
 
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         if not modified or "scrollable" in modified:
             if self.scrollable:
-                # 2003-06-26 ALB: change the "class name", to allow code generation
-                # for a wxScrolledWindow (see Node.write and common.class_names usage in xml_parse.py)
-                self._classname = 'EditScrolledWindow'
+                self.WX_CLASS = "wxScrolledWindow"
             else:
-                self._classname = self.__class__.__name__
-        PanelBase.properties_changed(self, modified)
-        ManagedBase.properties_changed(self, modified)
+                self.WX_CLASS = "wxPanel"
+        PanelBase._properties_changed(self, modified, actions)
+        ManagedBase._properties_changed(self, modified, actions)
 
     def get_properties(self, without=set()):
         # return list of properties to be written to XML file
         if not self.scrollable: without.add("scroll_rate")
         return ManagedBase.get_properties(self, without)
 
+    def _get_tooltip(self):
+        if self.parent.WX_CLASS in ("wxSplitterWindow", "wxNotebook"):
+            return "Add a sizer here; optionally, delete the panel and add e.g. a notebook, splitter or grid. "
+        return "Add a sizer here"
+
 
 class EditTopLevelPanel(PanelBase, TopLevelBase):
-    _is_toplevel_window = False  # avoid to appear in the "Top Window" property of the app
-
+    IS_TOPLEVEL_WINDOW = False  # avoid to appear in the "Top Window" property of the app
+    WX_CLASS = "wxPanel"
     PROPERTIES = TopLevelBase.PROPERTIES + PanelBase._PROPERTIES + TopLevelBase.EXTRA_PROPERTIES
 
-    def __init__(self, name, parent, id, klass='wxPanel', style='wxTAB_TRAVERSAL'):
-        TopLevelBase.__init__(self, name, klass, parent, id)
+    def __init__(self, name, parent, klass, style='wxTAB_TRAVERSAL'):
+        TopLevelBase.__init__(self, name, parent, klass)
         PanelBase.__init__(self, style)
         self.skip_on_size = False
 
@@ -249,7 +245,7 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
             win = wx.Frame( common.main, -1, misc.design_title(self.name), size=(400, 300), style=style )
             import os, compat
             icon = compat.wx_EmptyIcon()
-            xpm = os.path.join(config.icons_path, 'panel.xpm')
+            xpm = os.path.join(config.icons_path, 'panel.png')
             icon.CopyFromBitmap(misc.get_xpm_bitmap(xpm))
             win.SetIcon(icon)
             win.Bind(wx.EVT_CLOSE, self.hide_widget)  # CLOSE event of the frame, not the panel
@@ -257,45 +253,32 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
                 win.CentreOnScreen()
 
         if self.scrollable:
-            self.widget = wx.ScrolledWindow(win, self.id, style=0)
+            self.widget = wx.ScrolledWindow(win, wx.ID_ANY, style=0)
         else:
-            self.widget = wx.Panel(win, self.id, style=0)
+            self.widget = wx.Panel(win, wx.ID_ANY, style=0)
         self.widget.Bind(wx.EVT_ENTER_WINDOW, self.on_enter)
         self.widget.GetBestSize = self.get_widget_best_size
         #self.widget.SetSize = win.SetSize
 
     def create(self):
-        # XXX refactor this
-        size_p = self.properties['size']
-        oldval = size_p.get()
         super(EditTopLevelPanel, self).create()
-        if self.widget:
-            if not size_p.is_active() and self.top_sizer:
-                self.top_sizer.fit_parent()
-        if size_p.is_active() and oldval!=size_p.get():
-            size_p.set(oldval)
-            self.set_size()
+        if not self.check_prop("size") and self.children and self.children[0].IS_SIZER:
+            self.children[0].fit_parent()
+
+    def destroy_widget(self, level):
+        # handle containing frame
+        win = self.widget and self.widget.GetParent() or None
+        super(EditTopLevelPanel, self).destroy_widget(level)
+        if win is not None: win.Destroy()
 
     def hide_widget(self, event=None):
         # this is called from the context menu and from the EVT_CLOSE of the Frame
         self.widget.GetParent().Hide()
-        common.app_tree.expand(self.node, False)
+        common.app_tree.Collapse(self.item)
         self.design.update_label()
 
-    def set_name(self, name):
-        super(EditTopLevelPanel, self).set_name(name)
-        if self.widget:
-            self.widget.GetParent().SetTitle(misc.design_title(self.name))
-
-    def delete(self):
-        win = None
-        if self.widget:
-            win = self.widget.GetParent()
-        super(EditTopLevelPanel, self).delete()
-        if win is not None:
-            win.Destroy()
-
     def on_size(self, event):
+        # handle containing frame
         w, h = event.GetSize()
         if self.skip_on_size:
             self.skip_on_size = False
@@ -305,20 +288,18 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
         if self.widget.GetParent().GetClientSize() != (w, h):
             self.widget.GetParent().SetClientSize((w+2, h+2))
 
-    def properties_changed(self, modified):
+    def _properties_changed(self, modified, actions):
         if not modified or "scrollable" in modified:
             if self.scrollable:
-                # 2003-06-26 ALB: change the "class name", to allow code generation
-                # for a wxScrolledWindow (see Node.write and common.class_names usage in xml_parse.py)
-                self._classname = 'EditTopLevelScrolledWindow'
+                self.WX_CLASS = "wxScrolledWindow"
             else:
-                self._classname = self.__class__.__name__
+                self.WX_CLASS = "wxPanel"
         if not modified or "name" in modified:
             if self.widget:
                 self.widget.GetParent().SetTitle(misc.design_title(self.name))
 
-        PanelBase.properties_changed(self, modified)
-        TopLevelBase.properties_changed(self, modified)
+        PanelBase._properties_changed(self, modified, actions)
+        TopLevelBase._properties_changed(self, modified, actions)
 
     def get_properties(self, without=set()):
         # return list of properties to be written to XML file
@@ -326,77 +307,42 @@ class EditTopLevelPanel(PanelBase, TopLevelBase):
         return TopLevelBase.get_properties(self, without)
 
 
-def builder(parent, sizer, pos, number=[1]):
+def builder(parent, index):
     "factory function for EditPanel objects"
-    name = 'panel_%d' % number[0]
-    while common.app_tree.has_name(name):
-        number[0] += 1
-        name = 'panel_%d' % number[0]
+    name = parent.toplevel_parent.get_next_contained_name('panel_%d')
     with parent.frozen():
-        panel = EditPanel(name, parent, wx.NewId(), sizer, pos, style='')
-        node = Node(panel)
-        panel.node = node
-        panel.properties["proportion"].set(1)
-        panel.properties["flag"].set("wxEXPAND")
-        if parent.widget: panel.create()
-    common.app_tree.insert(node, sizer.node, pos-1)
-    #sizer.set_item(panel.pos, 1, wx.EXPAND)
+        editor = EditPanel(name, parent, index, style='')
+        editor.properties["proportion"].set(1)
+        editor.properties["flag"].set("wxEXPAND")
+        if parent.widget: editor.create()
+    return editor
 
 
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory to build EditPanel objects from a XML file"
-    from xml_parse import XmlParsingError
-    try:
-        name = attrs['name']
-    except KeyError:
-        raise XmlParsingError(_("'name' attribute missing"))
-    if not sizer or not sizeritem:
-        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
-    panel = EditPanel(name, parent, wx.NewId(), sizer, pos, style='')
-    #sizer.set_item(panel.pos, proportion=sizeritem.proportion, span=sizeritem.span, flag=sizeritem.flag, border=sizeritem.border)
-    node = Node(panel)
-    panel.node = node
-    if pos is None:
-        common.app_tree.add(node, sizer.node)
-    else:
-        common.app_tree.insert(node, sizer.node, pos-1)
-    return panel
+    return EditPanel(name, parent, index, '')
 
 
-def xml_toplevel_builder(attrs, parent, sizer, sizeritem, pos=None):
-    from xml_parse import XmlParsingError
-    try:
-        label = attrs['name']
-    except KeyError:
-        raise XmlParsingError(_("'name' attribute missing"))
-    panel = EditTopLevelPanel( label, parent, wx.NewId(), style='' )
-    node = Node(panel)
-    panel.node = node
-    common.app_tree.add(node)
-    return panel
+def xml_toplevel_builder(parser, base, name, parent, index):
+    return EditTopLevelPanel( name, parent, "Panel", '' )
 
 
 def initialize():
     "initialization function for the module: returns a wxBitmapButton to be added to the main palette"
+    common.widget_classes['EditPanel'] = EditPanel
     common.widgets['EditPanel'] = builder
     common.widgets_from_xml['EditPanel'] = xml_builder
 
-    #common.widgets['EditScrolledWindow'] = builder
+    common.widget_classes['EditScrolledWindow'] = EditPanel
     common.widgets_from_xml['EditScrolledWindow'] = xml_builder
 
+    common.widget_classes['EditTopLevelPanel'] = EditTopLevelPanel
     common.widgets_from_xml['EditTopLevelPanel'] = xml_toplevel_builder
+    common.widget_classes['EditTopLevelScrolledWindow'] = EditTopLevelPanel
     common.widgets_from_xml['EditTopLevelScrolledWindow'] = xml_toplevel_builder
-    from tree import WidgetTree
-    import os.path
-    icon = os.path.join(config.icons_path, 'panel.xpm')
-    WidgetTree.images['EditTopLevelPanel'] = icon
-    WidgetTree.images['EditScrolledWindow'] = icon
-    WidgetTree.images['EditTopLevelScrolledWindow'] = icon
 
     # these are for backwards compatibility (may be removed someday...)
     common.widgets_from_xml['SplitterPane'] = xml_builder
-    WidgetTree.images['SplitterPane'] = os.path.join( config.icons_path, 'panel.xpm' )
     common.widgets_from_xml['NotebookPane'] = xml_builder
-    WidgetTree.images['NotebookPane'] = os.path.join( config.icons_path, 'panel.xpm' )
-    return common.make_object_button('EditPanel', 'panel.xpm', tip='Add a Panel/ScrolledWindow')
+    return common.make_object_button('EditPanel', 'panel.png', tip='Add a Panel/ScrolledWindow')
 
