@@ -11,22 +11,19 @@ import wcodegen
 
 
 class PerlSliderGenerator(wcodegen.PerlWidgetCodeWriter):
-    tmpl = '%(name)s = %(klass)s->new(%(parent)s, %(id)s, %(value)s, ' \
-           '%(minValue)s, %(maxValue)s%(style)s);\n'
+    tmpl = '%(name)s = %(klass)s->new(%(parent)s, %(id)s, %(value)s, %(minValue)s, %(maxValue)s%(style)s);\n'
 
     def _prepare_tmpl_content(self, obj):
         wcodegen.PerlWidgetCodeWriter._prepare_tmpl_content(self, obj)
-        prop = obj.properties
-        self.tmpl_dict['value'] = prop.get('value', '0')
+        self.tmpl_dict['value'] = obj.value
         try:
-            minValue, maxValue = [s.strip() for s in prop['range'].split(',')]
+            minValue, maxValue = obj.properties["range"].get_tuple()
         except:
             minValue, maxValue = '0', '10'
         self.tmpl_dict['minValue'] = minValue
         self.tmpl_dict['maxValue'] = maxValue
         return
 
-# end of class PerlSliderGenerator
 
 
 def initialize():
