@@ -3,25 +3,26 @@ wxSpinCtrl objects
 
 @copyright: 2002-2007 Alberto Griggio
 @copyright: 2014-2016 Carsten Grohmann
-@copyright: 2016-2018 Dietmar Schwertberger
+@copyright: 2016-2020 Dietmar Schwertberger
 @license: MIT (see LICENSE.txt) - THIS PROGRAM COMES WITH NO WARRANTY
 """
 
 import wx
 from edit_windows import ManagedBase, EditStylesMixin
-from tree import Node
-import common, config, misc
+import time
+import common, misc
 import new_properties as np
 
 
 class EditSpinCtrl(ManagedBase, EditStylesMixin):
     "Class to handle wxSpinCtrl objects"
     # XXX unify with EditSpinButton?
+    WX_CLASS = 'wxSpinCtrl'
     _PROPERTIES = ["Widget", "range", "value", "style"]
     PROPERTIES = ManagedBase.PROPERTIES + _PROPERTIES + ManagedBase.EXTRA_PROPERTIES
 
-    def __init__(self, name, parent, id, sizer, pos):
-        ManagedBase.__init__(self, name, 'wxSpinCtrl', parent, id, sizer, pos)
+    def __init__(self, name, parent, index):
+        ManagedBase.__init__(self, name, parent, index)
         EditStylesMixin.__init__(self)
 
         # initialise instance properties
@@ -31,23 +32,24 @@ class EditSpinCtrl(ManagedBase, EditStylesMixin):
     def create_widget(self):
         mi,ma = self.properties["range"].get_tuple()
         if self.properties["value"].is_active():
-            self.widget = wx.SpinCtrl(self.parent.widget, self.id, min=mi, max=ma, initial=self.value)
+            self.widget = wx.SpinCtrl(self.parent_window.widget, wx.ID_ANY, min=mi, max=ma, initial=self.value)
         else:
-            self.widget = wx.SpinCtrl(self.parent.widget, self.id, min=mi, max=ma)
+            self.widget = wx.SpinCtrl(self.parent_window.widget, wx.ID_ANY, min=mi, max=ma)
 
-    def finish_widget_creation(self, sel_marker_parent=None, re_add=True):
-        ManagedBase.finish_widget_creation(self, sel_marker_parent, re_add)
+    def finish_widget_creation(self, level, sel_marker_parent=None):
+        ManagedBase.finish_widget_creation(self, level, sel_marker_parent)
         self.widget.Bind(wx.EVT_CHILD_FOCUS, self._on_set_focus)
         self.widget.Bind(wx.EVT_SET_FOCUS, self._on_set_focus)
         self.widget.Bind(wx.EVT_SPIN, self.on_set_focus)
 
     def _on_set_focus(self, event):
-        # don't set focused_widget during event, as this may cause crashes
-        if not misc.focused_widget is self:
+        # within a short time window, we ignore focus events as these seem due losing focus
+        if not misc.focused_widget is self and time.time()-misc.focused_time > 0.05:
+            # don't set focused_widget during event, as this may cause crashes; use delay instead
             misc.set_focused_widget(self, delayed=True)
         event.Skip()
 
-    def properties_changed(self, modified):  # from EditSlider
+    def _properties_changed(self, modified, actions):  # from EditSlider
         if not modified or "range" in modified and self.widget:
             mi,ma = self.properties["range"].get_tuple()
             self.widget.SetRange(mi, ma)
@@ -68,50 +70,32 @@ class EditSpinCtrl(ManagedBase, EditStylesMixin):
                 if self.widget:
                     self.widget.SetValue(value)
 
-        EditStylesMixin.properties_changed(self, modified)
-        ManagedBase.properties_changed(self, modified)
+        EditStylesMixin._properties_changed(self, modified, actions)
+        ManagedBase._properties_changed(self, modified, actions)
 
 
-def builder(parent, sizer, pos, number=[1]):
+def builder(parent, index):
     "factory function for EditSpinCtrl objects"
-    name = 'spin_ctrl_%d' % number[0]
-    while common.app_tree.has_name(name):
-        number[0] += 1
-        name = 'spin_ctrl_%d' % number[0]
+    name = parent.toplevel_parent.get_next_contained_name('spin_ctrl_%d')
     with parent.frozen():
-        spin = EditSpinCtrl(name, parent, wx.NewId(), sizer, pos)
-        spin.properties["style"].set_to_default()
-        spin.check_defaults()
-        node = Node(spin)
-        spin.node = node
-        if parent.widget: spin.create()
-    common.app_tree.insert(node, sizer.node, pos-1)
+        editor = EditSpinCtrl(name, parent, index)
+        editor.properties["style"].set_to_default()
+        editor.check_defaults()
+        if parent.widget: editor.create()
+    return editor
 
 
-def xml_builder(attrs, parent, sizer, sizeritem, pos=None):
+def xml_builder(parser, base, name, parent, index):
     "factory function to build EditSpinCtrl objects from a XML file"
-    from xml_parse import XmlParsingError
-    try:
-        name = attrs['name']
-    except KeyError:
-        raise XmlParsingError(_("'name' attribute missing"))
-    if sizer is None or sizeritem is None:
-        raise XmlParsingError(_("sizer or sizeritem object cannot be None"))
-    spin = EditSpinCtrl( name, parent, wx.NewId(), sizer, pos )
-    spin.properties["value"].set_active(False)
-    #sizer.set_item( spin.pos, proportion=sizeritem.proportion, flag=sizeritem.flag, border=sizeritem.border )
-    node = Node(spin)
-    spin.node = node
-    if pos is None:
-        common.app_tree.add(node, sizer.node)
-    else:
-        common.app_tree.insert(node, sizer.node, pos-1)
-    return spin
+    editor = EditSpinCtrl( name, parent, index )
+    editor.properties["value"].set_active(False)
+    return editor
 
 
 def initialize():
     "initialization function for the module: returns a wxBitmapButton to be added to the main palette"
+    common.widget_classes['EditSpinCtrl'] = EditSpinCtrl
     common.widgets['EditSpinCtrl'] = builder
     common.widgets_from_xml['EditSpinCtrl'] = xml_builder
 
-    return common.make_object_button('EditSpinCtrl', 'spin_ctrl.xpm')
+    return common.make_object_button('EditSpinCtrl', 'spin_ctrl.png')
