@@ -35,7 +35,6 @@ class LispCodeGenerator(wcodegen.LispWidgetCodeWriter):
 
         if obj.properties["separation"].is_active():
             append( '(wxToolBar_SetToolSeparation %s %s)\n' % (obj_name, obj.separation) )
-        append( '(wxToolBar_Realize %s)\n' % obj_name )
 
         return out
 
@@ -82,19 +81,23 @@ class LispCodeGenerator(wcodegen.LispWidgetCodeWriter):
             style += "|wxTB_HORIZONTAL"
             style = self.cn_f(style)
 
-        parent = self.format_widget_access(obj.parent)
+        parent = self.format_widget_access(obj.parent_window)
         obj_name = self.codegen._format_name(obj.name)
-        init = [ '\n\t;;; Tool Bar\n',
+        init = [ ';;; Tool Bar\n',
                  '(setf (slot-%s obj) (wxToolBar_Create %s -1 -1 -1 -1 -1 %s))\n' % (obj_name, parent, style),
-                      '(wxFrame_SetToolBar (slot-top-window obj) (slot-%s obj))\n' % obj_name ]
-        init.extend(self.get_init_code(obj))
-        init.append(';;; Tool Bar end\n')
-        return init, self.get_properties_code(obj), []
+                ] + self.get_init_code(obj) + self.get_properties_code(obj) + [
+                 '(wxFrame_SetToolBar (slot-top-window obj) (slot-%s obj))\n' % obj_name,
+                 '(wxToolBar_Realize %s)\n' % self.format_widget_access(obj),
+                 ';;; Tool Bar end\n']
+        return init, []
+
+    def get_layout_code(self, obj):
+        obj_name = '(slot-%s obj)' % self.codegen._format_name(obj.name)
+        return ['(wxToolBar_Realize %s)\n' % obj_name]
 
 
 
 def initialize():
     klass = 'wxToolBar'
     common.class_names['EditToolBar'] = klass
-    common.toplevels['EditToolBar'] = 1
     common.register('lisp', klass, LispCodeGenerator(klass) )#, 'tools', ToolsHandler)
